@@ -30,7 +30,6 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../src/cmd_gpsim.h"
 #include "../src/pic-processor.h"
-#include "../src/symbol_orb.h"
 #include "../src/operator.h"
 
 cmd_break c_break;
@@ -39,6 +38,7 @@ cmd_break c_break;
 #define EXECUTION   2
 #define WRITE       3
 #define READ        4
+#define REGCHANGE   5
 #define STK_OVERFLOW  7
 #define STK_UNDERFLOW 8
 #define WDT           9
@@ -49,6 +49,7 @@ static cmd_options cmd_break_options[] =
   {"e",   EXECUTION,    OPT_TT_BITFLAG},
   {"w",   WRITE,        OPT_TT_BITFLAG},
   {"r",   READ,         OPT_TT_BITFLAG},
+  {"ch",  REGCHANGE,    OPT_TT_BITFLAG},
   {"so",  STK_OVERFLOW, OPT_TT_BITFLAG},
   {"su",  STK_UNDERFLOW,OPT_TT_BITFLAG},
   {"wdt", WDT,          OPT_TT_BITFLAG},
@@ -56,11 +57,9 @@ static cmd_options cmd_break_options[] =
 };
 
 
-cmd_break::cmd_break(void)
+cmd_break::cmd_break()
+  : command("break", "br")
 { 
-  name = "break";
-  abbreviation = "br";
-
   brief_doc = string("Set a break point");
 
   long_doc = string ("The 'break' command can be used to examine or set breakpoints.\n"
@@ -71,15 +70,18 @@ cmd_break::cmd_break(void)
                      "    Halts when the address is executed, read, or written. The ADDRESS can be \n"
                      "    a symbol or a number. If the optional expr is specified, then it must\n"
                      "    evaluate to true before the simulation will halt. The optional message\n"
-		     "    allows a description to be associated with the break."
+		     "    allows a description to be associated with the break.\n"
                      "Register Memory breaks:\n"
-                     "  break r|w REGISTER [,expr] [,\"message\"]\n"
-                     "    Halts when 'REGISTER' is read or written and the optional expression\n"
-                     "    evaluates to true.\n"
-                     "  break r|w boolean_expression\n"
+                     "  break r|w|ch REGISTER [,expr] [,\"message\"]\n"
+                     "    Halts when 'REGISTER' is read, written, or changed\n"
+                     "    and the optional expression evaluates to true\n"
+                     "  break r|w|ch boolean_expression\n"
                      "    The boolean expression can only be of the form:\n"
 		     "       a) reg & mask == value\n"
 		     "       b) reg == value\n"
+                     "  - Note the 'ch' option is similar to the write option.\n"
+                     "    The change option evaluates the expression before and after\n"
+                     "    a register write and halts if the evaluation differs.\n"
                      "Cycle counter breaks:\n"
                      "  break c VALUE  [,\"message\"]\n"
                      "    Halts when the cycle counter reaches 'VALUE'.\n"
@@ -128,6 +130,8 @@ static gpsimObject::ObjectBreakTypes MapBreakActions(int co_value)
     return gpsimObject::eBreakRead;
   case WRITE:
     return gpsimObject::eBreakWrite;
+  case REGCHANGE:
+    return gpsimObject::eBreakChange;
   case EXECUTION:
     return gpsimObject::eBreakExecute;
   }
@@ -152,16 +156,20 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
   ExprList_itor ei = pEL->begin();
   Expression *pFirst = *ei;
   ++ei;
-  Expression *pSecond;
-  if(ei != pEL->end()) {
-    pSecond = *ei;
-    ++ei;
-  }
-  else {
-    pSecond = NULL;
-  }
+  Expression *pSecond  = (ei != pEL->end()) ? *ei++ : 0;
+
   Expression *pThird = (ei != pEL->end()) ? *ei : 0;
 
+  if (verbose) {
+    cout << "setting breakpoint:\n";
+    if (pFirst)
+      cout << " first expression" << pFirst->toString() << endl;
+    if (pSecond)
+      cout << " second expression" << pSecond->toString() << endl;
+    if (pThird)
+      cout << " third expression" << pThird->toString() << endl;
+  }
+
   LiteralString *pString=0;
   string m;
   if (pSecond) {
@@ -196,10 +204,10 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
   int bpn = pFirst ? pFirst->set_break(MapBreakActions(co->value), 
 				       (bLog ? gpsimObject::eActionLog : gpsimObject::eActionHalt),
 				       pSecond) : -1;
-  if (bpn == -1 && co->value!=CYCLE) {
-    GetUserInterface().DisplayMessage("break cannot be set on %s\n",
+  if (bpn == -1 && co->value!=CYCLE)
+    GetUserInterface().DisplayMessage("break cannot be set on '%s'\n",
       pFirst->toString().c_str());
-  }
+
   if (bpn<0) {
 
     // We failed to set a break point from the first expression. 
@@ -219,7 +227,10 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
   if (bpn>=0) {
     if (pString)
       get_bp().set_message(bpn, m);
-    get_bp().dump1(bpn);
+    if(verbose)get_bp().dump1(bpn); //RRR
+    if (dynamic_cast<LiteralInteger*>(pFirst))
+      delete pFirst;
+
   } else {
 
     delete pFirst;
@@ -227,6 +238,7 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
       delete pSecond;
   }
 
+  delete pEL;
   return bpn;
 
 }
@@ -340,3 +352,11 @@ unsigned int cmd_break::set_break(int bit_flag, bool bLog)
 
   return b;
 }
+ // attribute breakpoints 	 
+unsigned int cmd_break::set_break(gpsimObject *v)
+{
+  if (v)
+    v->set_break();
+
+  return MAX_BREAKPOINTS;
+}
