@@ -24,14 +24,12 @@ Boston, MA 02111-1307, USA.  */
 
 
 // forward references
-class SourceBrowserAsm_Window;     
+class SourceBrowserAsm_Window;
 class SourceBrowserParent_Window;
 class StatusBar_Window;
 class Value;
 class SourceBrowserParent_Window;
 
-// #if defined(NEW_SOURCE_BROWSER)
-
 class SourceWindow;
 
 enum eSourceFileType {
@@ -64,15 +62,18 @@ protected:
 // TextStyle
 //
 // A TextSyle is wrapper around a GtkTextTag and provides a simple way
-// to change the text foreground and background colors. Also, color 
+// to change the text foreground and background colors. Also, color
 // editing is supported.
 class TextStyle
 {
 public:
-
   TextStyle (const char *pName,
-	     const char *pFGColor,
-	     const char *pBGColor);
+             const char *pFGColor,
+             const char *pBGColor);
+
+  virtual ~TextStyle()
+  {
+  }
 
   GtkTextTag *tag() { return m_pTag; }
 
@@ -82,7 +83,7 @@ public:
   void setFG(GdkColor *pNewColor);
 
   virtual void doubleClickEvent(GtkTextIter *);
-  
+
   ColorHolder  mFG;  // Foreground color
   ColorHolder  mBG;  // Background color
 
@@ -98,7 +99,7 @@ public:
   void parseLine(const char*, int parseStyle);
 
   void addTagRange(TextStyle *,
-		   int start_index, int end_index);
+                   int start_index, int end_index);
   GtkTextBuffer *getBuffer();
   SourceBrowserParent_Window *m_pParent;
   FileContext   *m_pFC;
@@ -168,18 +169,18 @@ public:
   void setFont(const char *);
   void setSource();
   bool bHasSource();
-  
+
   FileContext *getFC();
 
   // callbacks
   static gint KeyPressHandler(GtkTextView *pView,
-                GdkEventKey *key, 
+                GdkEventKey *key,
                 SourceWindow *pSW);
   static gint ButtonPressHandler(GtkTextView *pView,
-                  GdkEventButton *pButton, 
+                  GdkEventButton *pButton,
                   SourceWindow *pSW);
   static gint ViewExposeEventHandler(GtkTextView *pView,
-                GdkEventExpose *pEvent, 
+                GdkEventExpose *pEvent,
                 SourceWindow *pSW);
 
 
@@ -198,10 +199,10 @@ class SearchDialog;
 class SourceWindow : public GUI_Object
 {
 public:
-  SourceWindow(GUI_Processor *gp, 
-	       SourceBrowserParent_Window *,
-	       bool bUseConfig,
-	       const char *newName=0);
+  SourceWindow(GUI_Processor *gp,
+               SourceBrowserParent_Window *,
+               bool bUseConfig,
+               const char *newName=0);
 
   virtual void Build();
   virtual void SetTitle();
@@ -250,7 +251,7 @@ public:
   GtkStyle *number_text_style;       // for numbers in .asm display
   GtkStyle *comment_text_style;      // for comments in .asm display
   GtkStyle *default_text_style;      // the rest
-    
+
   GdkFont  *symbol_font;             // for symbols in .asm display
   GdkFont  *label_font;              // for label in .asm display
   GdkFont  *instruction_font;        // for instruction in .asm display
@@ -278,7 +279,7 @@ private:
     int  line;                // Line within the page.
     GtkTextBuffer *pBuffer;   // Buffer containing the Program Counter
     GtkTextIter   iBegin;     // Start of where highlight begins
-    GtkTextIter   iEnd;       // End of highlight 
+    GtkTextIter   iEnd;       // End of highlight
   } mProgramCounter;
 
   // Popup Menu
@@ -289,7 +290,7 @@ private:
 
   // Callbacks
   static gint KeyPressHandler(GtkWidget *widget,
-                GdkEventKey *key, 
+                GdkEventKey *key,
                 SourceWindow *pSW);
   static int DeleteEventHandler(GtkWidget *widget,
                 GdkEvent  *event,
@@ -303,7 +304,7 @@ private:
 protected:
   void set_style_colors(const char *fg_color, const char *bg_color, GtkStyle **style);
   void addTagRange(NSourcePage *pPage, TextStyle *,
-		   int start_index, int end_index);
+                   int start_index, int end_index);
 
   // FIXME - change these items to list objects
   NSourcePage **pages;
@@ -363,7 +364,7 @@ public:
   int address;
   int pos;
   unsigned int index;           // gtktext index to start of line
-private:  
+private:
   unsigned int line;            // line number, first line eq. 0
   GtkWidget *break_widget;      // breakpoint widget on this line.
   GtkWidget *canbreak_widget;   // 'can break' widget on this line.
@@ -384,15 +385,15 @@ class BreakPointList {
 class PixmapObject {
 public:
   PixmapObject(void)
-   { 
+   {
      mask = 0;
      pixmap = 0;
      widget = 0;
    }
 
   void CreateFromXPM(GdkWindow *window,
-		      GdkColor *transparent_color,
-		      gchar **data);
+                      GdkColor *transparent_color,
+                      gchar **data);
 
   GdkBitmap *mask;
   GdkPixmap *pixmap;
@@ -425,12 +426,12 @@ class SourcePage
     notebook_child = 0;
     notebook = 0;
   }
-    
+
   void Close(void);
 };
 
 //
-// The Source Assembler Browser 
+// The Source Assembler Browser
 //
 class SourceBrowserAsm_Window :public  SourceBrowser_Window
 {
@@ -467,7 +468,7 @@ class SourceBrowserAsm_Window :public  SourceBrowser_Window
   GtkWidget *popup_menu;
 
   BreakPointInfo *menu_data;  // used by menu callbacks
-    
+
   GdkBitmap *pc_mask;
   GdkBitmap *bp_mask;
   GdkBitmap *canbp_mask;
@@ -481,7 +482,7 @@ class SourceBrowserAsm_Window :public  SourceBrowser_Window
   GtkStyle *number_text_style;       // for numbers in .asm display
   GtkStyle *comment_text_style;      // for comments in .asm display
   GtkStyle *default_text_style;      // the rest
-    
+
   GdkFont  *symbol_font;             // for symbols in .asm display
   GdkFont  *label_font;              // for label in .asm display
   GdkFont  *instruction_font;        // for instruction in .asm display
@@ -506,9 +507,6 @@ class SourceBrowserAsm_Window :public  SourceBrowser_Window
   SourceBrowserParent_Window *parent;
 
   SourceBrowserAsm_Window(GUI_Processor *gp,char* new_name);
-#if ! defined(NEW_SOURCE_BROWSER)
-  virtual void Build(void);
-#endif
   virtual void SelectAddress(int address);
   virtual void SelectAddress(Value *);
   virtual void SetPC(int address);
@@ -574,7 +572,7 @@ class SourceBrowserOpcode_Window : public SourceBrowser_Window
   GdkColor normal_pm_bg_color;
   GdkColor breakpoint_color;
 
-  char **column_titles; //
+  gchar **column_titles; //
   int  columns;         //
 
   GtkWidget *notebook;
@@ -589,7 +587,7 @@ class SourceBrowserOpcode_Window : public SourceBrowser_Window
   // one byte/cell,
   // two bytes/cell MSB first
   // two bytes/cell LSB first
-    
+
   unsigned int *memory;
 
   SourceBrowserOpcode_Window(GUI_Processor *gp);
@@ -605,17 +603,12 @@ class SourceBrowserOpcode_Window : public SourceBrowser_Window
 
 
 
-#if defined(NEW_SOURCE_BROWSER)
 #define SOURCE_WINDOW SourceWindow
-#else
-#define SOURCE_WINDOW SourceBrowserAsm_Window
-#endif
-
 
 
 //
 // The Source Browser Child window.
-// 
+//
 // The gui supports "context" debugging, where a context
 // may be code written for interrupt routines, non-interrupt
 // routines, high versus low interrupt priorities, etc. Each
@@ -642,8 +635,6 @@ class SourceBrowserParent_Window : public GUI_Object
   SOURCE_WINDOW *getChild(int);
   list<SOURCE_WINDOW *> children;
 
-// #if defined(NEW_SOURCE_BROWSER)
-
   ProgramMemoryAccess *pma;      // pointer to the processor's pma.
 
   GtkTextTagTable *getTagTable() { return mpTagTable; }
@@ -677,7 +668,7 @@ class SourceBrowserParent_Window : public GUI_Object
 
   // FIXME - change these items to list objects
   SourceBuffer **ppSourceBuffers;
-// #endif
+
 };
 
 
