@@ -71,9 +71,9 @@ Boston, MA 02111-1307, USA.  */
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
+#include "../config.h"
 #include "../src/gpsim_time.h"
 #include "stimuli.h"
-#include "../config.h"
 #include "../src/pic-ioports.h"
 #include "../src/symbol.h"
 #include "../src/trace.h"
@@ -107,12 +107,22 @@ namespace ExtendedStimuli {
   };
 
 
+  class PulseInitial : public Float
+  {
+  public:
+    PulseInitial(PulseGen *pParent, const char *_name, const char * desc, double voltage);
+    virtual void set(double);
+  private:
+    PulseGen *m_pParent;
+    double    m_voltage;
+  };
+
 
   //----------------------------------------------------------------------
   //----------------------------------------------------------------------
   PulseAttribute::PulseAttribute(PulseGen *pParent,
-				 const char *_name, const char * desc,
-				 double voltage)
+                                 const char *_name, const char * desc,
+                                 double voltage)
     : Integer(_name,0,desc), m_pParent(pParent), m_voltage(voltage)
   {
   }
@@ -128,7 +138,7 @@ namespace ExtendedStimuli {
 
   //----------------------------------------------------------------------
   PulsePeriodAttribute::PulsePeriodAttribute(PulseGen *pParent,
-					     const char *_name, const char * desc)
+                                             const char *_name, const char * desc)
     : Integer(_name,0,desc), m_pParent(pParent)
   {
   }
@@ -139,6 +149,21 @@ namespace ExtendedStimuli {
     m_pParent->update_period();
   }
 
+  //----------------------------------------------------------------------
+  //----------------------------------------------------------------------
+  PulseInitial::PulseInitial(PulseGen *pParent,
+                             const char *_name, const char * desc,
+                             double voltage)
+    : Float(_name,0,desc), m_pParent(pParent), m_voltage(voltage)
+  {
+  }
+
+  void PulseInitial::set(double d)
+  {
+    Float::set(d);
+    m_pParent->update();
+  }
+
   //----------------------------------------------------------------------
   // StimulusBase
   //----------------------------------------------------------------------
@@ -146,7 +171,7 @@ namespace ExtendedStimuli {
     : Module(_name,_desc)
   {
     // Default module attributes.
-    initializeAttributes();
+    //initializeAttributes();
 
     // The I/O pin
     m_pin = new IO_bi_directional((name() + ".pin").c_str());
@@ -185,7 +210,6 @@ namespace ExtendedStimuli {
   Module *PulseGen::construct(const char *new_name)
   {
     PulseGen *pPulseGen = new PulseGen(new_name);
-    pPulseGen->create_iopin_map();
     return pPulseGen;
   }
 
@@ -199,27 +223,35 @@ Pulse Generator\n\
  .set - time when the pulse will drive high\n\
  .clear - time when the pulse will drive low\n\
  .period - time the pulse stream is repeated\n\
+ .initial - initial pin voltage\n\
 "), m_future_cycle(0), m_start_cycle(0)
   {
     // Attributes for the pulse generator.
     m_set = new PulseAttribute(this, "set","r/w cycle time when ouput will be driven high", 5.0);
     m_clear = new PulseAttribute(this, "clear","r/w cycle time when ouput will be driven low",0.0);
     m_period = new PulsePeriodAttribute(this,
-					"period","r/w cycle time to specify pulse stream repeat rate");
+                                        "period","r/w cycle time to specify pulse stream repeat rate");
+    m_init = new PulseInitial(this, "initial","initial I/O pin voltage", 5.0);
 
-    add_attribute(m_set);
-    add_attribute(m_clear);
-    add_attribute(m_period);
+    addSymbol(m_set);
+    addSymbol(m_clear);
+    addSymbol(m_period);
+    addSymbol(m_init);
 
     sample_iterator = samples.end();
+    create_iopin_map();
   }
 
   PulseGen::~PulseGen()
   {
-    delete m_pin;
+    removeSymbol(m_set);
+    removeSymbol(m_clear);
+    removeSymbol(m_period);
+    removeSymbol(m_init);
     delete m_set;
     delete m_clear;
     delete m_period;
+    delete m_init;
   }
 
   //----------------------------------------------------------------------
@@ -228,7 +260,9 @@ Pulse Generator\n\
   {
     //guint64 currCycle = get_cycles().get();
 
+
     if (sample_iterator != samples.end()) {
+      m_future_cycle = 0;
 
       double d;
       (*sample_iterator).v->get(d);
@@ -240,8 +274,8 @@ Pulse Generator\n\
       // then we're done.
 
       if (sample_iterator == samples.end()  &&
-	  m_period->getVal() == 0)
-	return;
+          m_period->getVal() == 0)
+        return;
 
       // If this is a periodic pulse stream and either
       //     a) we reached the end of the sequence
@@ -249,10 +283,10 @@ Pulse Generator\n\
       // then
       //   start the stream over.
       if (m_period->getVal() && ((sample_iterator == samples.end()  ||
-				  (*sample_iterator).time > m_period->getVal()))) {
+                                  (*sample_iterator).time > m_period->getVal()))) {
 
-	sample_iterator = samples.begin();
-	m_start_cycle += m_period->getVal();
+        sample_iterator = samples.begin();
+        m_start_cycle += m_period->getVal();
       }
 
       m_future_cycle = m_start_cycle + (*sample_iterator).time;
@@ -289,7 +323,7 @@ Pulse Generator\n\
   // job is to compare a list element to a reference value. Stroustrop
   // calls this function a predicate.
 
-  static guint64 current_cycle=0;
+  static gint64 current_cycle=0;
   static bool cycleIsInFuture(ValueStimulusData &data_point)
   {
     return data_point.time > current_cycle;
@@ -313,8 +347,12 @@ Pulse Generator\n\
   void PulseGen::update()
   {
 
-    if (samples.empty())
+    if (samples.empty()) {
+      double d;
+      m_init->get(d);
+      m_pin->putState(d > 2.5);
       return;  // There are no samples
+    }
 
     current_cycle = get_cycles().get();
 
@@ -331,14 +369,14 @@ Pulse Generator\n\
       // If the next sample *is* the second one then we've been here before
       // (and that means we've already handled the first sample)
       if (sample_iterator == si)
-	return;
+        return;
 
       if (si == samples.end()) {
-	si = samples.begin();
-	sample_iterator = si;
-	double d;
-	(*si).v->get(d);
-	m_pin->putState(d > 2.5);
+        si = samples.begin();
+        sample_iterator = si;
+        double d;
+        (*si).v->get(d);
+        m_pin->putState(d > 2.5);
       }
 
       sample_iterator = si;
@@ -400,7 +438,7 @@ Pulse Generator\n\
       (*si).v->get(d);
       sOut << "  {0x" << (*si).time << ',' << d << '}';
       if (si == sample_iterator)
-	sOut << " <-- Next at cycle 0x" << (m_start_cycle + (*si).time);
+        sOut << " <-- Next at cycle 0x" << (m_start_cycle + (*si).time);
       ++si;
     }
     sOut << ends;
@@ -430,8 +468,8 @@ Pulse Generator\n\
   };
   //=1024;
   FileNameAttribute::FileNameAttribute(FileStimulus *pParent,
-				       const char *_name,
-				       const char * _desc)
+                                       const char *_name,
+                                       const char * _desc)
     : String(_name,"",_desc), m_Parent(pParent), m_pFile(0)
   {
 
@@ -469,7 +507,7 @@ File Stimulus\n\
     // Attributes for the pulse generator.
     m_file = new FileNameAttribute(this, "file","name of file or pipe supplying data");
 
-    add_attribute(m_file);
+    addSymbol(m_file);
 
   }
 
@@ -498,7 +536,7 @@ File Stimulus\n\
     float v;
 
     //fscanf(m_file,"%" PRINTF_INT64_MODIFIER "i %g",&t, &v);
-    sscanf(cP,"%" PRINTF_INT64_MODIFIER "i %g",&t, &v);
+    sscanf(cP,"%" PRINTF_GINT64_MODIFIER "i %g",&t, &v);
 
     cout << "  read 0x" << hex << t << "," << v << endl;
 
@@ -531,7 +569,7 @@ File Stimulus\n\
     virtual void set(gint64);
   private:
     Register *m_replaced;
-    const int InvalidAddress;
+    const unsigned int InvalidAddress;
   };
 
   RegisterAddressAttribute::RegisterAddressAttribute(Register *pReg,const char *_name, const char * desc)
@@ -549,13 +587,13 @@ File Stimulus\n\
     if (pcpu && m_replaced) {
 
       if (m_replaced->address != InvalidAddress)
-	pcpu->rma.removeRegister(m_replaced->address,m_replaced);
+        pcpu->rma.removeRegister(m_replaced->address,m_replaced);
 
       m_replaced->set_cpu(pcpu);
 
       m_replaced->address = i & 0xffffffff;
       if (!pcpu->rma.insertRegister(m_replaced->address,m_replaced))
-	  m_replaced->address = InvalidAddress;
+          m_replaced->address = InvalidAddress;
 
       gint64 insertAddress = m_replaced->address;
 
@@ -581,42 +619,92 @@ File Stimulus\n\
 
   //----------------------------------------------------------------------
   //----------------------------------------------------------------------
-
-  Module *PortStimulus::construct(const char *new_name)
+  class PortPullupRegister : public sfr_register
   {
-    PortStimulus *pPortStimulus = new PortStimulus(new_name);
-    pPortStimulus->create_iopin_map();
-    return pPortStimulus;
+  public:
+    PortPullupRegister(const char *_name,
+                       PicPortRegister *_port,
+                       unsigned int enableMask);
+    ~PortPullupRegister() {}
+    virtual void put(unsigned int new_value);
+  private:
+    PicPortRegister *m_port;
+    unsigned int m_EnableMask;
+  };
+
+  PortPullupRegister::PortPullupRegister(const char *_name,
+                                         PicPortRegister *_port,
+                                         unsigned int enableMask)
+    : sfr_register(0,_name,"Port Pullup"),m_port(_port),m_EnableMask(enableMask)
+  {
+    new_name(_name);
+    value = RegisterValue(0,~enableMask);
+  }
+
+  void PortPullupRegister::put(unsigned int new_value)
+  {
+    get_trace().raw(write_trace.get() | value.data);
+    unsigned int diff = (value.data ^ new_value) & m_EnableMask;
+    value.data = new_value;
+    if (diff && m_port) {
+
+      for (unsigned int i=1,j=0; diff && i; i<<=1, j++)
+        if (diff & i)
+          m_port->getPin(j)->update_pullup(((value.data & i) !=0) ? '1':'0',true);
+
+      m_port->updatePort();
+    }
   }
 
   //----------------------------------------------------------------------
   //----------------------------------------------------------------------
 
-  PortStimulus::PortStimulus(const char *_name)
+  Module *PortStimulus::construct8(const char *new_name)
+  {
+    return new PortStimulus(new_name,8);
+  }
+  Module *PortStimulus::construct16(const char *new_name)
+  {
+    return new PortStimulus(new_name,16);
+  }
+  Module *PortStimulus::construct32(const char *new_name)
+  {
+    return new PortStimulus(new_name,32);
+  }
+
+  //----------------------------------------------------------------------
+  //----------------------------------------------------------------------
+
+  PortStimulus::PortStimulus(const char *_name, int nPins)
     : Module(_name, "\
 Port Stimulus\n\
  Attributes:\n\
  .port - port name\n\
  .tris - tris name\n\
  .lat  - latch name\n\
-")
+ .pullup  - pullup name\n\
+"),
+      m_nPins(nPins)
   {
-    mPort  = new PicPortRegister((name()+".port").c_str(),8,0xff);
-    mTris  = new PicTrisRegister((name()+".tris").c_str(),mPort);
-    mLatch = new PicLatchRegister((name()+".lat").c_str(),mPort);
-    mLatch->setEnableMask(0xff);
+    mPort   = new PicPortRegister(0,(name()+".port").c_str(),"",m_nPins,(1<<m_nPins)-1);
+    mTris   = new PicTrisRegister(0,(name()+".tris").c_str(),"",mPort,true,(1<<m_nPins)-1);
+    mLatch  = new PicLatchRegister(0,(name()+".lat").c_str(),"",mPort,(1<<m_nPins)-1);
+    mPullup = new PortPullupRegister((name()+".pullup").c_str(),mPort,(1<<m_nPins)-1);
 
     mPortAddress = new RegisterAddressAttribute(mPort, "portAdr","Port register address");
     mTrisAddress = new RegisterAddressAttribute(mTris, "trisAdr","Tris register address");
     mLatchAddress = new RegisterAddressAttribute(mLatch, "latAdr","Latch register address");
+    mPullupAddress = new RegisterAddressAttribute(mPullup, "pullupAdr","Pullup register address");
 
-    get_symbol_table().add_register(mPort);
-    get_symbol_table().add_register(mTris);
-    get_symbol_table().add_register(mLatch);
+    addSymbol(mPort);
+    addSymbol(mTris);
+    addSymbol(mLatch);
+    addSymbol(mPullup);
 
-    add_attribute(mPortAddress);
-    add_attribute(mTrisAddress);
-    add_attribute(mLatchAddress);
+    addSymbol(mPortAddress);
+    addSymbol(mTrisAddress);
+    addSymbol(mLatchAddress);
+    addSymbol(mPullupAddress);
 
     // FIXME - probably want something better than the generic module trace
 
@@ -624,10 +712,19 @@ Port Stimulus\n\
     get_trace().allocateTraceType(mMTT);
 
     buildTraceType(mPort, mMTT->type());
-    buildTraceType(mTris, mMTT->type() + (4<<8));
-    buildTraceType(mLatch, mMTT->type() + (8<<8));
+    buildTraceType(mTris, mMTT->type() + (4<<16));
+    buildTraceType(mLatch, mMTT->type() + (8<<16));
+    buildTraceType(mPullup, mMTT->type() + (12<<16));
+
+    create_iopin_map();
 
   }
+
+  PortStimulus::~PortStimulus()
+  {
+    printf("~PortStimulus\n");
+  }
+
   //----------------------------------------------------------------------
   //----------------------------------------------------------------------
 
@@ -640,17 +737,28 @@ Port Stimulus\n\
   void PortStimulus::create_iopin_map()
   {
 
-    create_pkg(8);
+    create_pkg(m_nPins);
+    char pinNumber[3];
+
+    for (int i=0; i<m_nPins; i++) {
+      int p = i+1;
+      if (p<10) {
+        pinNumber[0] = p+'0';
+        pinNumber[1] = 0;
+      } else {
+        pinNumber[0] = (p/10)+'0';
+        pinNumber[1] = (p%10)+'0';
+        pinNumber[2] = 0;
+      }
 
-    for (int i=0; i<8; i++) {
-      char pinNumber = '1'+i;
       IO_bi_directional *ppin;
 
-      ppin = new IO_bi_directional((name() + ".p" + pinNumber).c_str());
+      ppin = new IO_bi_directional_pu((name() + ".p" + pinNumber).c_str());
       ppin->update_direction(IOPIN::DIR_OUTPUT,true);
       assign_pin(i+1, mPort->addPin(ppin,i));
     }
   }
 
 
+
 } // end of namespace ExtendedStimuli
