@@ -1,5 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
+   Copyright (C) 2009 Roy R. Rankin
 
 This file is part of gpsim.
 
@@ -31,6 +32,7 @@ class TMRL;
 class TMRH;
 class TMR2;
 class CCPRL;
+class CCPCON;
 class ADCON0;
 class PIR_SET;
 class InterruptSource;
@@ -56,7 +58,7 @@ public:
   bool  pwm_mode;
   unsigned int pwm_value;
 
-  CCPRH();
+  CCPRH(Processor *pCpu, const char *pName, const char *pDesc=0);
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
@@ -69,16 +71,18 @@ class CCPRL : public sfr_register
 public:
 
   CCPRH  *ccprh;
+  CCPCON *ccpcon;
   TMRL   *tmrl;
 
   void put(unsigned int new_value);
   void capture_tmr();
-  void start_compare_mode();
+  void start_compare_mode(CCPCON *ref=0);
   void stop_compare_mode();
+  bool test_compare_mode();
   void start_pwm_mode();
   void stop_pwm_mode();
   void assign_tmr(TMRL *ptmr);
-  CCPRL();
+  CCPRL(Processor *pCpu, const char *pName, const char *pDesc=0);
 };
 
 
@@ -128,10 +132,12 @@ public:
   void pwm_match(int new_state);
   void put(unsigned int new_value);
   char getState();
+  bool test_compare_mode();
 
-  void setCrosslinks(CCPRL *, PIR_SET *, TMR2 *);
+  void setCrosslinks(CCPRL *, PIR *, TMR2 *);
   void setADCON(ADCON0 *);
-  CCPCON();
+  CCPCON(Processor *pCpu, const char *pName, const char *pDesc=0);
+  ~CCPCON();
 
   void setIOpin(PinModule *);
 private:
@@ -144,13 +150,14 @@ private:
   int   edges;
 
   CCPRL   *ccprl;
-  PIR_SET *pir_set;
+  PIR     *pir;
   TMR2    *tmr2;
   ADCON0  *adcon0;
 
-
-
 };
+
+
+class TMR1_Freq_Attribute;
 //---------------------------------------------------------
 // T1CON - Timer 1 control register
 
@@ -166,12 +173,15 @@ enum
   T1OSCEN = 1<<3,
   T1CKPS0 = 1<<4,
   T1CKPS1 = 1<<5,
-  T1RD16  = 1<<6
+  T1RD16  = 1<<6,
+  TMR1GE  = 1<<6,  // TMR1 Gate Enable used if TMR1L::setGatepin() has been called
+  T1GINV  = 1<<7
 };
 
   TMRL  *tmrl;
+  TMR1_Freq_Attribute *freq_attribute;
 
-  T1CON();
+  T1CON(Processor *pCpu, const char *pName, const char *pDesc=0);
 
   unsigned int get();
 
@@ -192,6 +202,22 @@ enum
     {
       return(value.get() & TMR1ON);
     }
+  unsigned int get_t1oscen()
+    {
+      return(value.get() & T1OSCEN);
+    }
+  unsigned int get_tmr1GE()
+    {
+      return(value.get() & TMR1GE);
+    }
+  unsigned int get_t1GINV()
+    {
+      return(value.get() & T1GINV);
+    }
+  unsigned int get_t1sync()
+    {
+      return(value.get() & T1SYNC);
+    }
   virtual void put(unsigned int new_value);
 
 };
@@ -209,36 +235,44 @@ public:
   unsigned int get();
   virtual unsigned int get_value();
 
-  TMRH();
+  TMRH(Processor *pCpu, const char *pName, const char *pDesc=0);
 
 };
 
+class TMR1CapComRef;
+
 class TMRL : public sfr_register, public TriggerObject, public SignalSink
 {
 public:
 
   TMRH  *tmrh;
   T1CON *t1con;
-  CCPCON *ccpcon;
 
   unsigned int 
     prescale,
     prescale_counter,
     break_value,
-    compare_value,
     value_16bit;         /* Low and high concatenated */
 
+  double ext_scale;
+
+  TMR1CapComRef * compare_queue;
+
   guint64
     synchronized_cycle,
-    last_cycle,
     future_cycle;
+  gint64 last_cycle;  // last_cycle can be negative for small cycle counts
 
-  bool compare_mode;
 
   virtual void callback();
   virtual void callback_print();
 
-  TMRL();
+  TMRL(Processor *pCpu, const char *pName, const char *pDesc=0);
+  ~TMRL();
+
+  void set_ext_scale();
+
+  virtual void release();
 
   virtual void put(unsigned int new_value);
   virtual unsigned int get();
@@ -251,12 +285,30 @@ public:
   virtual void clear_timer();
   virtual void setSinkState(char);
   virtual void setIOpin(PinModule *);
+  virtual void setGatepin(PinModule *);
+  virtual void IO_gate(bool);
+  virtual void compare_gate(bool);
   virtual void setInterruptSource(InterruptSource *);
+  virtual void sleep();
+  virtual void wake();
+
+  void set_compare_event ( unsigned int value, CCPCON *host );
+  void clear_compare_event ( CCPCON *host );
+
+  void set_T1GSS(bool arg);
+
 protected:
   virtual void increment();   // Used when TMR1 is attached to an external clock
 private:
   char m_cState;
+  bool m_GateState;		// Only changes state if setGatepin() has been called
+  bool m_compare_GateState;
+  bool m_io_GateState;
   bool m_bExtClkEnabled;
+  bool m_sleeping;
+  bool m_t1gss;			// T1 gate source
+				// true - IO pin controls gate, 
+				// false - compare controls gate
   InterruptSource *m_Interrupt;
 };
 
@@ -271,7 +323,7 @@ public:
 
   TMR2 *tmr2;
 
-  PR2();
+  PR2(Processor *pCpu, const char *pName, const char *pDesc=0);
   void put(unsigned int new_value);
 
 };
@@ -296,7 +348,7 @@ enum
 
   TMR2 *tmr2;
 
-  T2CON();
+  T2CON(Processor *pCpu, const char *pName, const char *pDesc=0);
 
   inline unsigned int get_t2ckps0()
     {
@@ -381,7 +433,7 @@ public:
 
   virtual void callback();
   virtual void callback_print();
-  TMR2();
+  TMR2(Processor *pCpu, const char *pName, const char *pDesc=0);
 
   void put(unsigned int new_value);
   unsigned int get();
