@@ -30,6 +30,7 @@ This file originated by J.R. Heisey
 #include <iostream>
 #include <sstream>
 #include <algorithm>
+#include <cstdio>
 
 #include "value.h"
 #include "expr.h"
@@ -78,6 +79,7 @@ This file originated by J.R. Heisey
 */
 class IIndexedCollection : public Value {
 public:
+  IIndexedCollection(const char *pName, const char *pDesc, int iAddressRadix = 10);
   IIndexedCollection(int iAddressRadix = 10);
   virtual unsigned int GetSize() = 0;
   virtual Value &GetAt(unsigned int uIndex, Value *pValue=0) = 0;
@@ -179,6 +181,7 @@ public:
       sName = pName;
       sprintf(szIndex, "[%d]", uIndex + m_uLower);
       sName.append(szIndex);
+      // Hmm... Do we really want to create new object for every array entry?
       m_Array.push_back(new _CT(sName.c_str(), stDefValue, pDesc));
     }
   }
@@ -212,16 +215,20 @@ public:
     }
   }
 
-  void SetAt(unsigned int uIndex, _CT *pValue) {
-    if((uIndex + 1 - m_uLower) < m_Array.size() && uIndex <= m_uLower) {
+  void SetAt(unsigned int uIndex, _CT *pValue)
+  {
+    if((uIndex + 1 - m_uLower) < m_Array.size() && uIndex >= m_uLower) {
+
       _ST stValue;
       pValue->get(stValue);
+
       _CT * pElement = static_cast<_CT*>(m_Array.at(uIndex - m_uLower));
-      pElement->set(stValue);
-    }
-    else {
+      if (pElement)
+	pElement->set(stValue);
+
+    } else {
+
       char szIndex[10];
-      // throw something?
       sprintf(szIndex, "%d", uIndex);
       string sMsg("invalid array index of ");
       sMsg.append(szIndex);
@@ -269,7 +276,7 @@ public:
     // This loop examines every element's value and records
     // the value in aValue. aList is used to record an
     // appropriate label for one or more elements.
-    for(it = itLastEqualed = m_Array.begin(); it != itEnd; it++) {
+    for(it = itLastEqualed = m_Array.begin(); it != itEnd; ++it) {
       ostringstream sIndex;
       if(*(_CT*)(*itLastEqualed) != *(_CT*)(*it)) {
         PushValue(iFirstIndex, iCurrentIndex - 1,
@@ -296,7 +303,7 @@ public:
   unsigned int GetUpperBound() {
      return (unsigned int)m_Array.size();
   }
-
+  /* --- This isn't used and I (Scott) don't understand the intent...
   void SetLowerBound(unsigned int uIndex) {
     m_uLower = uIndex;
     // Now iterate through the elements and change
@@ -316,6 +323,7 @@ public:
     }
     free(pNameBuffer);
   }
+  */
 
 protected:
   void push_back(_CT *p) {
