@@ -61,14 +61,21 @@ void WarnModeAttribute::get(bool &b)
 // SafeModeAttribute
 //========================================================================
 
-SafeModeAttribute::SafeModeAttribute(Processor *_cpu) :
-  Boolean(false) ,cpu(_cpu)
+SafeModeAttribute::SafeModeAttribute(Processor *_cpu) 
+  :
+  Boolean("SafeMode",
+          " Model the processor's specification when true. Model the actual\n"
+          " processor when false (e.g. TRIS instruction for mid range PICs\n"
+          " will emit a warning if SafeMode is true).",
+          false),
+  cpu(_cpu)
 {
-  new_name("SafeMode");
-  set_description(" Model the processor's specification when true. Model the actual\n"
-		  " processor when false (e.g. TRIS instruction for mid range PICs\n"
-		  " will emit a warning if SafeMode is true).");
 }
+
+SafeModeAttribute::~SafeModeAttribute()
+{
+}
+
 void SafeModeAttribute::set(Value *v)
 {
   Boolean::set(v);
@@ -143,7 +150,7 @@ public:
   CycleCounterAttribute() :
     Integer(0) 
   {
-    m_bClearableSymbol = false;
+    //m_bClearableSymbol = false;
     new_name("cycles");
     set_description(" Simulation time in terms of cycles.");
   }
@@ -185,7 +192,7 @@ public:
   GUIUpdateRateAttribute() :
     Integer(0) 
   {
-    m_bClearableSymbol = false;
+    //m_bClearableSymbol = false;
     new_name("sim.gui_update_rate");
     set_description(" Specifies the number of cycles between gui updates");
   }
@@ -209,24 +216,40 @@ Integer *verbosity;
 //########################################################################
 void init_attributes()
 {
-  
   // Define internal simulator attributes .
   verbosity = new Integer("sim.verbosity",1,"gpsim's verboseness 0=nothing printed 0xff=very verbose");
-  verbosity->setClearableSymbol(false);
-  get_symbol_table().Initialize();
-  get_symbol_table().add(verbosity);
-  get_symbol_table().add(new CycleCounterAttribute());
-  stop_watch.init();
+  globalSymbolTable().addSymbol(verbosity);
+  globalSymbolTable().addSymbol(new CycleCounterAttribute());
+  stop_watch = new StopWatch;
 #ifdef HAVE_GUI
-  get_symbol_table().add(new GUIUpdateRateAttribute());
+  globalSymbolTable().addSymbol(new GUIUpdateRateAttribute());
 #endif
 
-  get_symbol_table().add_constant("POR_RESET",  POR_RESET, false);    // Power-on reset
-  get_symbol_table().add_constant("WDT_RESET",  WDT_RESET, false);    // Watch Dog timer timeout reset
-  get_symbol_table().add_constant("IO_RESET",   IO_RESET, false);     // I/O pin reset
-  get_symbol_table().add_constant("SOFT_RESET", SOFT_RESET, false);   // Software initiated reset
-  get_symbol_table().add_constant("BOD_RESET",  BOD_RESET, false);    // Brown out detection reset
-  get_symbol_table().add_constant("SIM_RESET",  SIM_RESET, false);    // Simulation Reset
-  get_symbol_table().add_constant("MCLR_RESET", MCLR_RESET, false);   // MCLR (Master Clear) Reset
+  globalSymbolTable().addSymbol(new Integer("POR_RESET",  POR_RESET));    // Power-on reset
+  globalSymbolTable().addSymbol(new Integer("WDT_RESET",  WDT_RESET));    // Watch Dog timer timeout reset
+  globalSymbolTable().addSymbol(new Integer("IO_RESET",   IO_RESET));     // I/O pin reset
+  globalSymbolTable().addSymbol(new Integer("SOFT_RESET", SOFT_RESET));   // Software initiated reset
+  globalSymbolTable().addSymbol(new Integer("BOD_RESET",  BOD_RESET));    // Brown out detection reset
+  globalSymbolTable().addSymbol(new Integer("SIM_RESET",  SIM_RESET));    // Simulation Reset
+  globalSymbolTable().addSymbol(new Integer("MCLR_RESET", MCLR_RESET));   // MCLR (Master Clear) Reset
 
 }
+
+void destroy_attributes()
+{
+  globalSymbolTable().deleteSymbol("SourcePath");
+  globalSymbolTable().deleteSymbol("sim.verbosity");
+  globalSymbolTable().deleteSymbol("cycles");
+#ifdef HAVE_GUI
+  globalSymbolTable().deleteSymbol("sim.gui_update_rate");
+#endif
+  globalSymbolTable().deleteSymbol("POR_RESET");
+  globalSymbolTable().deleteSymbol("WDT_RESET");
+  globalSymbolTable().deleteSymbol("IO_RESET");
+  globalSymbolTable().deleteSymbol("SOFT_RESET");
+  globalSymbolTable().deleteSymbol("BOD_RESET");
+  globalSymbolTable().deleteSymbol("SIM_RESET");
+  globalSymbolTable().deleteSymbol("MCLR_RESET");
+
+  delete stop_watch;
+}
