@@ -70,6 +70,7 @@ public:
   Breakpoint_Instruction(Processor *new_cpu, 
 			 unsigned int new_address, 
 			 unsigned int bp);
+  virtual ~Breakpoint_Instruction();
 
   virtual enum INSTRUCTION_TYPES isa() {return BREAKPOINT_INSTRUCTION;};
   virtual void execute();
@@ -86,6 +87,7 @@ class Notify_Instruction : public Breakpoint_Instruction
 		     unsigned int address, 
 		     unsigned int bp, 
 		     TriggerObject *cb);
+  virtual ~Notify_Instruction();
   virtual enum INSTRUCTION_TYPES isa() {return NOTIFY_INSTRUCTION;};
   virtual void execute();
   virtual char const * bpName() { return "Notify Execution"; }
@@ -172,6 +174,7 @@ class RegisterAssertion : public Breakpoint_Instruction
 		    unsigned int _regValue,
 		    bool bPostAssertion=false
 		    );
+  virtual ~RegisterAssertion();
 
   virtual void execute();
   virtual void print();
@@ -286,7 +289,8 @@ public:
 		     unsigned int, 
 		     unsigned int,
 		     TriggerObject *f = 0);
-  int set_breakpoint(TriggerObject *,Expression *pExpr=0);
+
+  int set_breakpoint(TriggerObject *,Processor *, Expression *pExpr=0);
 
   int set_execution_break(Processor *cpu, unsigned int address, Expression *pExpr=0);
   int set_notify_break(Processor *cpu, 
@@ -318,6 +322,7 @@ public:
             unsigned int op,
             unsigned int value,
             unsigned int mask=0xff);
+  int set_change_break(Processor *cpu, unsigned int register_number);
   int set_break(gpsimObject::ObjectBreakTypes bt,
 		gpsimObject::ObjectActionTypes at,
 		Register *,
@@ -436,11 +441,10 @@ class BreakpointRegister : public Register, public TriggerObject
 {
 public:
 
-
+  // FIXME: why 2 constructors?
   BreakpointRegister(Processor *, TriggerAction *, Register *);
-
-  BreakpointRegister(Processor *, int, int );
   BreakpointRegister(Processor *, TriggerAction *, int, int );
+  virtual ~BreakpointRegister();
 
   virtual REGISTER_TYPES isa() {return BP_REGISTER;};
   virtual string &name() const;
@@ -465,11 +469,10 @@ public:
   virtual void print();
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
+  virtual void invokeAction();
   virtual void clear();
-
+  virtual void takeAction();
 protected:
-  BreakpointRegister();
-
 };
 
 class BreakpointRegister_Value : public BreakpointRegister
@@ -480,25 +483,6 @@ public:
   unsigned int m_uDefRegMask;
   string m_sOperator;
 
-  BreakpointRegister_Value()
-  { 
-    break_value = 0;
-    break_mask = 0;
-  }
-
-  BreakpointRegister_Value(Processor *_cpu, 
-        int _repl, 
-        int bp, 
-        unsigned int bv, 
-        unsigned int bm );
-
-  BreakpointRegister_Value(Processor *_cpu, 
-         TriggerAction *pTA,
-        int _repl, 
-        int bp, 
-        unsigned int bv, 
-        unsigned int bm );
-
   BreakpointRegister_Value(Processor *_cpu, 
         int _repl, 
         int bp, 
@@ -506,6 +490,8 @@ public:
         unsigned int _operator,
         unsigned int bm );
 
+  virtual ~BreakpointRegister_Value();
+
   typedef bool (*PFNISBREAKCONDITION)(unsigned int uRegValue,
       unsigned int uRegMask, unsigned int uRegTestValue);
   PFNISBREAKCONDITION m_pfnIsBreak;
@@ -533,21 +519,18 @@ public:
   static bool IsLessThenEqualsBreakCondition(unsigned int uRegValue,
       unsigned int uRegMask, unsigned int uRegTestValue);
 
-  virtual void invokeAction();
   virtual void print();
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
 };
 
 
-class Break_register_read : public BreakpointRegister, TriggerAction
+class Break_register_read : public BreakpointRegister
 {
 public:
 
-  Break_register_read(Processor *_cpu, int _repl, int bp ):
-    BreakpointRegister(_cpu, _repl,bp ) { 
-    set_action(this);
-  }
+  Break_register_read(Processor *_cpu, int _repl, int bp );
+  virtual ~Break_register_read();
 
   virtual unsigned int get();
   virtual RegisterValue getRV();
@@ -558,18 +541,15 @@ public:
 
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
-  virtual void invokeAction();
-  // TriggerAction overrides
-  virtual void action();
+  virtual void takeAction();
 };
 
-class Break_register_write : public BreakpointRegister, TriggerAction
+class Break_register_write : public BreakpointRegister
 {
 public:
-  Break_register_write(Processor *_cpu, int _repl, int bp ):
-    BreakpointRegister(_cpu,_repl,bp ) { 
-    set_action(this);
-  }
+  Break_register_write(Processor *_cpu, int _repl, int bp );
+  virtual ~Break_register_write();
+
   virtual void put(unsigned int new_value);
   virtual void putRV(RegisterValue rv);
   virtual void setbit(unsigned int bit_number, bool new_value);
@@ -577,27 +557,35 @@ public:
 
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
-  virtual void invokeAction();
-  // TriggerAction overrides
-  virtual void action();
+  virtual void takeAction();
 };
 
-class Break_register_read_value : public BreakpointRegister_Value, TriggerAction
+class Break_register_change : public BreakpointRegister
 {
 public:
-//  Break_register_read_value(){ };
-  Break_register_read_value(Processor *_cpu, 
-			    int _repl, 
-			    int bp, 
-			    unsigned int bv, 
-			    unsigned int bm );
+  Break_register_change(Processor *_cpu, int _repl, int bp );
+  virtual ~Break_register_change();
 
+  virtual void put(unsigned int new_value);
+  virtual void putRV(RegisterValue rv);
+  virtual void setbit(unsigned int bit_number, bool new_value);
+  virtual char const * bpName() { return "register change"; }
+
+  virtual int  printTraced(Trace *pTrace, unsigned int tbi,
+			   char *pBuf, int szBuf);
+  virtual void takeAction();
+};
+
+class Break_register_read_value : public BreakpointRegister_Value
+{
+public:
   Break_register_read_value(Processor *_cpu, 
 			    int _repl, 
 			    int bp, 
 			    unsigned int bv,
-          unsigned int _operator,
+                            unsigned int _operator,
 			    unsigned int bm );
+  virtual ~Break_register_read_value();
 
   virtual unsigned int get();
   virtual RegisterValue getRV();
@@ -608,25 +596,19 @@ public:
 
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
-  // TriggerAction overrides
-  virtual void action();
+  virtual void takeAction();
 };
 
-class Break_register_write_value : public BreakpointRegister_Value, TriggerAction
+class Break_register_write_value : public BreakpointRegister_Value
 {
 public:
-  Break_register_write_value(Processor *_cpu, 
-			     int _repl, 
-			     int bp, 
-			     unsigned int bv, 
-			     unsigned int bm );
-
   Break_register_write_value(Processor *_cpu, 
 			     int _repl, 
 			     int bp, 
 			     unsigned int bv, 
 			     unsigned int _operator,
 			     unsigned int bm );
+  virtual ~Break_register_write_value();
 
   virtual void put(unsigned int new_value);
   virtual void putRV(RegisterValue rv);
@@ -635,8 +617,7 @@ public:
 
   virtual int  printTraced(Trace *pTrace, unsigned int tbi,
 			   char *pBuf, int szBuf);
-  // TriggerAction overrides
-  virtual void action();
+  virtual void takeAction();
 };
 
 class CommandAssertion : public Breakpoint_Instruction
@@ -650,6 +631,7 @@ public:
                    const char *_command,
                    bool bPostAssertion
                    );
+  virtual ~CommandAssertion();
 
   virtual void execute();
   virtual void print();
@@ -660,17 +642,15 @@ private:
   char *command;
 };
 
+// FIXME -- the log classes need to be deprecated - use the action class to perform logging instead.
 class Log_Register_Write : public Break_register_write
 {
  public:
 
   Log_Register_Write(Processor *_cpu, int _repl, int bp ):
     Break_register_write(_cpu,_repl,bp ) { };
-  //virtual void put(unsigned int new_value);
-  //virtual void putRV(RegisterValue rv);
-  //virtual void setbit(unsigned int bit_number, bool new_value);
   virtual char const * bpName() { return "log register write"; }
-  virtual void action();
+  virtual void takeAction();
 };
 
 class Log_Register_Read : public Break_register_read
@@ -679,19 +659,12 @@ public:
   Log_Register_Read(Processor *_cpu, int _repl, int bp ):
     Break_register_read(_cpu,_repl,bp ) { };
   virtual char const * bpName() { return "log register read"; }
-  virtual void action();
+  virtual void takeAction();
 };
 
 class Log_Register_Read_value : public  Break_register_read_value
 {
 public:
-
-  Log_Register_Read_value(Processor *_cpu, 
-			  int _repl, 
-			  int bp, 
-			  unsigned int bv, 
-			  unsigned int bm ) :
-    Break_register_read_value(_cpu,  _repl, bp, bv, bm ) { };
   Log_Register_Read_value(Processor *_cpu, 
 			  int _repl, 
 			  int bp, 
@@ -701,20 +674,12 @@ public:
     Break_register_read_value(_cpu,  _repl, bp, bv, _operator, bm ) { };
 
   virtual char const * bpName() { return "log register read value"; }
-  virtual void action();
+  virtual void takeAction();
 };
 
 class Log_Register_Write_value : public Break_register_write_value
 {
 public:
-
-  Log_Register_Write_value(Processor *_cpu, 
-			   int _repl, 
-			   int bp, 
-			   unsigned int bv, 
-			   unsigned int bm ) :
-    Break_register_write_value(_cpu,  _repl, bp, bv, bm ) { };
-
   Log_Register_Write_value(Processor *_cpu, 
 			     int _repl, 
 			     int bp, 
@@ -722,7 +687,7 @@ public:
 			     unsigned int _operator,
 			     unsigned int bm ) :
     Break_register_write_value(_cpu,  _repl, bp, bv, _operator,bm ) { };
-  virtual void action();
+  virtual void takeAction();
 
 };
 
