@@ -34,6 +34,9 @@ Boston, MA 02111-1307, USA.  */
 #include "stimuli.h"
 #include "12bit-processors.h"
 
+class PicPortRegister;
+class PicTrisRegister;
+class PicLatchRegister;
 
 
 class P16C54 : public  _12bit_processor
@@ -45,6 +48,13 @@ public:
   PicPortRegister  *m_portb;
   PicTrisRegister  *m_trisb;
 
+#ifdef USE_PIN_MODULE_FOR_TOCKI
+  PinModule    *m_tocki;
+#else
+  PicPortRegister  *m_tocki;
+  PicTrisRegister  *m_trist0;
+#endif
+  
   virtual PROCESSOR_TYPE isa(){return _P16C54_;};
   virtual void create_symbols();
 
@@ -56,11 +66,12 @@ public:
 
   virtual void option_new_bits_6_7(unsigned int bits) {}
 
-  P16C54();
+  P16C54(const char *_name=0, const char *desc=0);
+  virtual ~P16C54();
   void create();
   virtual void create_iopin_map();
 
-  static Processor *construct();
+  static Processor *construct(const char *name);
   virtual void tris_instruction(unsigned int tris_register);
 
   virtual unsigned int fsr_valid_bits()
@@ -92,11 +103,12 @@ public:
 
   virtual void create_sfr_map();
 
-  P16C55();
+  P16C55(const char *_name=0, const char *desc=0);
+  virtual ~P16C55();
   virtual void create();
   virtual void create_iopin_map();
 
-  static Processor *construct();
+  static Processor *construct(const char *name);
   virtual void tris_instruction(unsigned int tris_register);
 
 };
@@ -111,9 +123,9 @@ public:
   virtual unsigned int register_memory_size() const { return 0x20; };
   virtual unsigned int config_word_address() const {return 0xFFF;};
 
-  P16C56();
+  P16C56(const char *_name=0, const char *desc=0);
 
-  static Processor *construct();
+  static Processor *construct(const char *name);
 
 };
 
