@@ -1,5 +1,6 @@
 /*
    Copyright (C) 1998 Scott Dattalo
+   Copyright (C) 2009 Roy R. Rankin
 
 This file is part of gpsim.
 
@@ -31,12 +32,12 @@ Boston, MA 02111-1307, USA.  */
 #include "pic-ioports.h"
 #include "interface.h"
 #include "p16x6x.h"
-#include "p16f62x.h"
+//#include "p16f62x.h"
 
 #include "xref.h"
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("%s:%d",__FILE__,__LINE__); printf arg; }
+#define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
@@ -72,8 +73,7 @@ public:
   }
   char getState()
   {
-    // return m_register ? 
-    //  (((m_register->getDriving()&m_bitMask)!=0)?'1':'0') : 'Z';
+
     char r = m_register ? (((m_register->getDriving()&m_bitMask)!=0)?'1':'0') : 'Z';
     /**/
     Dprintf(("PicSignalSource::getState() %s  bitmask:0x%x state:%c\n",
@@ -82,6 +82,10 @@ public:
     /**/
     return r;
   }
+  void release()
+  {
+    delete this;
+  }
 private:
   PortRegister *m_register;
   unsigned int  m_bitMask;
@@ -91,28 +95,36 @@ private:
 
 //------------------------------------------------------------------------
 
-PicPortRegister::PicPortRegister(const char *port_name,
+PicPortRegister::PicPortRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                 /*const char *port_name,*/
 				 unsigned int numIopins, 
 				 unsigned int enableMask)
-  : PortRegister(numIopins, false), m_tris(0)
+  : PortRegister(pCpu, pName, pDesc,numIopins, false), m_tris(0)
 {
-  new_name(port_name);
-  PortRegister::setEnableMask(enableMask);
+  setEnableMask(enableMask);
 }
 
 class PicSignalControl : public SignalControl
 {
 public:
-  PicSignalControl(Register *_reg, unsigned int bitPosition)
+  PicSignalControl(PicTrisRegister *_reg, unsigned int bitPosition)
     : m_register(_reg), m_bitMask(1<<bitPosition)
   {
   }
-  char getState()
+  ~PicSignalControl()
+  {
+  }
+
+  virtual char getState()
   {
     return m_register ? m_register->get3StateBit(m_bitMask) : '?';
   }
+  virtual void release()
+  {
+    delete this;
+  }
 private:
-  Register *m_register;
+  PicTrisRegister *m_register;
   unsigned int m_bitMask;
 };
 
@@ -129,10 +141,16 @@ void PicPortRegister::setTris(PicTrisRegister *new_tris)
 }
 //------------------------------------------------------------------------
 
-PicTrisRegister::PicTrisRegister(const char *tris_name, PicPortRegister *_port)
-  : sfr_register(),m_port(_port)
+PicTrisRegister::PicTrisRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                 /*const char *tris_name,*/
+				 PicPortRegister *_port,
+                                 bool bIgnoreWDTResets,
+				 unsigned int enableMask)
+  : sfr_register(pCpu, pName, pDesc),
+    m_port(_port),
+    m_EnableMask(enableMask),
+    m_bIgnoreWDTResets(bIgnoreWDTResets)
 {
-  new_name(tris_name);
   if (m_port)
     m_port->setTris(this);
 }
@@ -150,14 +168,35 @@ unsigned int PicTrisRegister::get()
   return value.data;
 }
 
+void PicTrisRegister::setEnableMask(unsigned int enableMask)
+{
+  m_EnableMask = enableMask;
+}
+
+char PicTrisRegister::get3StateBit(unsigned int bitMask)
+{
+  RegisterValue rv = getRV_notrace();
+  unsigned int enabled = bitMask & m_EnableMask;
+  if (!enabled)
+    return '1';
+
+  return (rv.init&enabled) ? '?' : (rv.data&enabled ? '1': '0');
+
+}
+void PicTrisRegister::reset(RESET_TYPE r)
+{
+  if (!(m_bIgnoreWDTResets && r==WDT_RESET))
+    putRV(por_value);
+}
+
+
 //------------------------------------------------------------------------
 
-PicPSP_TrisRegister::PicPSP_TrisRegister(const char *tris_name, PicPortRegister *_port)
-  : sfr_register(),m_port(_port)
+PicPSP_TrisRegister::PicPSP_TrisRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                         /*const char *tris_name, */
+                                         PicPortRegister *_port, bool bIgnoreWDTResets)
+  : PicTrisRegister(pCpu, pName, pDesc,_port,bIgnoreWDTResets)
 {
-  new_name(tris_name);
-  if (m_port)
-    m_port->setTris((PicTrisRegister *)this);
 }
 // If not in PSPMODE, OBF and IBF are always clear
 // When in PSPMODE, OBF and IBF can only be cleared by reading and writing
@@ -170,9 +209,9 @@ void PicPSP_TrisRegister::put(unsigned int new_value)
 
   trace.raw(write_trace.get() | value.data);
   if (! (new_value & PSP::PSPMODE))
-	fixed = 0;
-   else
-	fixed = value.data & mask;
+    fixed = 0;
+  else
+    fixed = value.data & mask;
 
   value.data = (new_value & ~mask) | fixed;
   if (m_port)
@@ -199,18 +238,30 @@ unsigned int PicPSP_TrisRegister::get(void)
 
 
 
-PicPortBRegister::PicPortBRegister(const char *port_name, unsigned int numIopins, unsigned int enableMask)
-  : PicPortRegister(port_name, numIopins, enableMask),
+PicPortBRegister::PicPortBRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                   INTCON *pIntcon,
+				   unsigned int numIopins, 
+				   unsigned int enableMask)
+  : PicPortRegister(pCpu, pName, pDesc, numIopins, enableMask),
     m_bRBPU(false),
-    m_bIntEdge(false)
+    m_bIntEdge(true), 
+    m_bsRBPU(0),
+    m_pIntcon(pIntcon)
 {
+  assert(m_pIntcon);
 }
 
+PicPortBRegister::~PicPortBRegister()
+{
+  delete m_bsRBPU;
+}
+//------------------------------------------------------------------------
+
 void PicPortBRegister::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.data);
 
-  cpu14->intcon->set_rbif(false);
+  m_pIntcon->set_rbif(false);
 
 //  unsigned int diff = mEnableMask & (new_value ^ value.data);
 //RRR  if(diff) {
@@ -228,9 +279,9 @@ void PicPortBRegister::put(unsigned int new_value)
 
 unsigned int PicPortBRegister::get()
 {
-  cpu14->intcon->set_rbif(false);
+  m_pIntcon->set_rbif(false);
 
-  return rvDrivenValue.data;
+  return mOutputMask & rvDrivenValue.data;
 }
 
 //------------------------------------------------------------------------
@@ -239,20 +290,51 @@ unsigned int PicPortBRegister::get()
 
 void PicPortBRegister::setbit(unsigned int bit_number, char new3State)
 {
-  Dprintf(("PicPortBRegister::setbit() bit=%d,val=%c\n",bit_number,new3State));
+  Dprintf(("PicPortBRegister::setbit() bit=%d,val=%c bIntEdge=%d\n",bit_number,new3State, m_bIntEdge));
 
+  // interrupt bit 0 on specified edge 
   bool bNewValue = new3State=='1' || new3State=='W';
   if (bit_number == 0 && (((rvDrivenValue.data&1)==1)!=m_bIntEdge) 
       && (bNewValue == m_bIntEdge))
-    cpu14->intcon->set_intf(true);
+    m_pIntcon->set_intf(true);
 
 
+  RegisterValue lastDrivenValue = rvDrivenValue;
   PortRegister::setbit(bit_number, new3State);
 
+ // interrupt and exit sleep level change top 4 bits on input
   unsigned int bitMask = (1<<bit_number) & 0xF0;
 
-  if ( (drivingValue ^ rvDrivenValue.data) & m_tris->get() & bitMask )
-    cpu14->intcon->set_rbif(true);
+  if ( (lastDrivenValue.data ^ rvDrivenValue.data) & m_tris->get() & bitMask ) {
+    cpu_pic->exit_sleep();
+    m_pIntcon->set_rbif(true);
+  }
+}
+
+class RBPUBitSink : public BitSink
+{
+  PicPortBRegister *m_pPortB;
+public:
+  RBPUBitSink(PicPortBRegister *pPortB) 
+    : m_pPortB(pPortB) 
+  {}
+    
+  void setSink(bool b)
+  {
+    if (m_pPortB)
+      m_pPortB->setRBPU(b);
+  }
+};
+
+void PicPortBRegister::assignRBPUSink(unsigned int bitPos, sfr_register *pSFR)
+{
+  if (pSFR && !m_bsRBPU) {
+    m_bsRBPU = new RBPUBitSink(this);
+    if (!pSFR->assignBitSink(bitPos, m_bsRBPU)) {
+      delete m_bsRBPU;
+      m_bsRBPU = 0;
+    }
+  }
 }
 
 void PicPortBRegister::setRBPU(bool bNewRBPU)
@@ -275,14 +357,41 @@ void PicPortBRegister::setIntEdge(bool bNewIntEdge)
   m_bIntEdge = bNewIntEdge;
 }
 
-
-PicPSP_PortRegister::PicPSP_PortRegister(const char *port_name,
-                                 unsigned int numIopins,
-                                 unsigned int enableMask)
-  : PortRegister(numIopins, false), m_tris(0), m_psp(0)
+// set_rbif involves RBIF,RBIE in INTCON which are the same bits as GPIF,GPIE
+void PicPortGRegister::setbit(unsigned int bit_number, char new3State)
 {
-  new_name(port_name);
-  PortRegister::setEnableMask(enableMask);
+  // interrupt bit 2 on specified edge 
+  bool bNewValue = new3State=='1' || new3State=='W';
+  if (bit_number == 2 && (((rvDrivenValue.data&1)==1)!=m_bIntEdge) 
+      && (bNewValue == m_bIntEdge))
+    m_pIntcon->set_intf(true);
+
+    RegisterValue lastDrivenValue = rvDrivenValue;
+    PortRegister::setbit(bit_number, new3State);
+
+ // interrupt and exit sleep for level change on bits where IOC set
+    bool bitMask = m_pIoc->get() & (1 << bit_number);
+
+   if (verbose)
+       printf("PicPortGRegister::setbit() bit=%d,val=%c IOC_bit=%x\n",bit_number,new3State, bitMask);
+
+    if ( (lastDrivenValue.data ^ rvDrivenValue.data) & m_tris->get() & bitMask ) {
+      cpu_pic->exit_sleep();
+      m_pIntcon->set_rbif(true);
+    }
+}
+void PicPortGRegister::setIntEdge(bool bNewIntEdge)
+{
+  m_bIntEdge = bNewIntEdge;
+}
+
+PicPSP_PortRegister::PicPSP_PortRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                         /*const char *port_name,*/
+                                         unsigned int numIopins,
+                                         unsigned int enableMask)
+  : PortRegister(pCpu, pName, pDesc,numIopins, false), m_tris(0), m_psp(0)
+{
+  setEnableMask(enableMask);
 }
 
 void PicPSP_PortRegister::put(unsigned int new_value)
@@ -331,10 +440,13 @@ void PicPSP_PortRegister::setTris(PicTrisRegister *new_tris)
 //------------------------------------------------------------------------
 // Latch Register
 
-PicLatchRegister::PicLatchRegister(const char *_name, PortRegister *_port)
-  : m_port(_port), m_EnableMask(0xff)
+PicLatchRegister::PicLatchRegister(Processor *pCpu, const char *pName, const char *pDesc,
+                                   /*const char *_name, */
+				   PortRegister *_port,
+				   unsigned int enableMask)
+  : sfr_register(pCpu, pName, pDesc),
+    m_port(_port), m_EnableMask(enableMask)
 {
-  new_name(_name);
 }
 
 void PicLatchRegister::put(unsigned int new_value)
