@@ -24,9 +24,11 @@ Boston, MA 02111-1307, USA.  */
 #include "trace.h"
 #include "pir.h"
 #include "intcon.h"
+#include "processor.h"
 
-PIR::PIR(INTCON *_intcon, PIE *_pie, int _valid_bits)
-  : intcon(_intcon),pie(_pie),valid_bits(_valid_bits)  ,writable_bits(0)
+PIR::PIR(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie, int _valid_bits)
+  : sfr_register(pCpu,pName,pDesc),
+    intcon(_intcon),pie(_pie),ipr(0),valid_bits(_valid_bits),writable_bits(0)
 {
 }
 
@@ -39,10 +41,10 @@ void PIR::put(unsigned int new_value)
 
   trace.raw(write_trace.get() | value.get());
   //trace.register_write(address,value.get());
-  value.put(new_value & writable_bits | value.get() & ~writable_bits);
+  value.put((new_value & writable_bits) | (value.get() & ~writable_bits));
 
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 
 }
 
@@ -57,6 +59,11 @@ void PIR::set_pie(PIE *_pie)
   pie = _pie;
 }
 
+void PIR::set_ipr(sfr_register *_ipr)
+{
+  ipr = _ipr;
+}
+
 void PIR::setInterrupt(unsigned int bitMask)
 {
   put(value.get() | bitMask);
@@ -66,18 +73,26 @@ void PIR::setInterrupt(unsigned int bitMask)
 void PIR::setPeripheralInterrupt()
 {
   if (intcon)
-    intcon->peripheral_interrupt();
+    intcon->peripheral_interrupt ( ipr && (value.get() & valid_bits & ipr->value.get() & pie->value.get()) );
 }
-bool PIR::interrupt_status()
+
+int PIR::interrupt_status()
 {
   assert(pie);
-  if( value.get() & valid_bits & pie->value.get())
-    return true;
-
-  return false;
+  if ( ipr )
+  {
+    int result = 0;
+    if ( value.get() & valid_bits & pie->value.get() & ~(ipr->value.get()) )
+        result |= 1;
+    if ( value.get() & valid_bits & pie->value.get() & ipr->value.get() )
+        result |= 2;
+    return result;
+  }
+  else
+    return ( value.get() & valid_bits & pie->value.get() ) ? 1 : 0;
 }
 
-
+//========================================================================
 InterruptSource::InterruptSource(PIR *_pir, unsigned int bitMask)
   : m_pir(_pir), m_bitMask(bitMask)
 {
@@ -92,14 +107,17 @@ void InterruptSource::Trigger()
 }
 
 
-
+void InterruptSource::release()
+{
+  delete this;
+}
 
 
 
 //------------------------------------------------------------------------
 
-PIR1v1::PIR1v1(INTCON *_intcon, PIE *_pie)
-  : PIR(_intcon, _pie,0)
+PIR1v1::PIR1v1(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
 {
   // Even though TXIF is a valid bit, it can't be written by the PIC
   // source code.  Its state reflects whether the usart txreg is full
@@ -120,7 +138,7 @@ void PIR1v1::set_txif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | TXIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v1::clear_txif(void)
@@ -134,7 +152,7 @@ void PIR1v1::set_rcif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | RCIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v1::clear_rcif(void)
@@ -148,7 +166,7 @@ void PIR1v1::set_cmif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | CMIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v1::set_eeif(void)
@@ -156,12 +174,12 @@ void PIR1v1::set_eeif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | EEIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 //------------------------------------------------------------------------
 //
-PIR1v2::PIR1v2(INTCON *_intcon, PIE *_pie)
-  : PIR(_intcon, _pie,0)
+PIR1v2::PIR1v2(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
 {
   // Even though TXIF is a valid bit, it can't be written by the PIC
   // source code.  Its state reflects whether the usart txreg is full
@@ -181,7 +199,7 @@ void PIR1v2::set_txif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | TXIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v2::set_pspif(void)
@@ -189,14 +207,14 @@ void PIR1v2::set_pspif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | PSPIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 void PIR1v2::set_sspif(void)
 {
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | SSPIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v2::clear_txif(void)
@@ -210,7 +228,7 @@ void PIR1v2::set_rcif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | RCIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 
 void PIR1v2::clear_rcif(void)
@@ -220,15 +238,47 @@ void PIR1v2::clear_rcif(void)
 }
 
 //------------------------------------------------------------------------
-PIR2v1::PIR2v1(INTCON *_intcon, PIE *_pie)
-  : PIR(_intcon, _pie,0)
+
+PIR1v3::PIR1v3(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
+{
+  valid_bits = TMR1IF | ADIF | CMIF | EEIF;
+  writable_bits = TMR1IF | ADIF | CMIF | EEIF;
+}
+
+void PIR1v3::set_cmif(void)
+{
+  trace.raw(write_trace.get() | value.get());
+  value.put(value.get() | CMIF);
+  if( value.get() & pie->value.get() )
+    setPeripheralInterrupt();
+}
+
+void PIR1v3::set_eeif(void)
+{
+  trace.raw(write_trace.get() | value.get());
+  value.put(value.get() | EEIF);
+  if( value.get() & pie->value.get() )
+    setPeripheralInterrupt();
+}
+void PIR1v3::set_adif(void)
+{
+  trace.raw(write_trace.get() | value.get());
+  value.put(value.get() | ADIF);
+  if( value.get() & pie->value.get() )
+    setPeripheralInterrupt();
+}
+
+//------------------------------------------------------------------------
+PIR2v1::PIR2v1(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
 {
   valid_bits = CCP2IF;
   writable_bits = valid_bits;
 }
 //------------------------------------------------------------------------
-PIR2v2::PIR2v2(INTCON *_intcon, PIE *_pie)
-  : PIR(_intcon, _pie,0)
+PIR2v2::PIR2v2(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
 {
   valid_bits = ECCP1IF | TMR3IF | LVDIF | BCLIF | EEIF | CMIF;
   writable_bits = valid_bits;
@@ -239,25 +289,25 @@ void PIR2v2::set_cmif(void)
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | CMIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 void PIR2v2::set_eeif(void)
 {
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | EEIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 void PIR2v2::set_bclif(void)
 {
   trace.raw(write_trace.get() | value.get());
   value.put(value.get() | BCLIF);
   if( value.get() & pie->value.get() )
-    intcon->peripheral_interrupt();
+    setPeripheralInterrupt();
 }
 //------------------------------------------------------------------------
-PIR3v2::PIR3v2(INTCON *_intcon, PIE *_pie)
-  : PIR(_intcon, _pie,0)
+PIR3v2::PIR3v2(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon, PIE *_pie)
+  : PIR(pCpu,pName,pDesc,_intcon, _pie,0)
 {
   valid_bits = RXB0IF | RXB1IF | TXB0IF | TXB1IF | TXB2IF | ERRIF |
     WAKIF | IRXIF;
