@@ -39,52 +39,12 @@ class symbol;
 /* typedefs */
 typedef list<Value*> SymbolList_t;
 typedef list<string> StringList_t;
-
-class Pin_t {
-public:
-  enum {
-    ePackageBased = 0x0001,
-    ePortBased    = 0x0002,
-    eActiveProc   = 0x0004,
-  };
-  Pin_t(int iFlags, Value * pPin) {
-    m_iFlags = iFlags;
-    m_sModuleName = NULL;
-    m_sPin = pPin;
-    m_sPort = NULL;
-  }
-  Pin_t(int iFlags, Value * pModuleName, Value * pPin) {
-    m_iFlags = iFlags;
-    m_sModuleName = pModuleName;
-    m_sPin = pPin;
-    m_sPort = NULL;
-  }
-  Pin_t(int iFlags, Value * pModuleName, Value *pPort, Value * pPin) {
-    m_iFlags = iFlags;
-    m_sModuleName = pModuleName;
-    m_sPin = pPin;
-    m_sPort = pPort;
-  }
-  stimulus *GetStimulus();
-  IOPIN * GetIOPin();
-  Value * GetValue();
-  Value * m_sModuleName;
-  Value * m_sPin;
-  Value * m_sPort;
-  int     m_iFlags;
-};
-
-typedef list<Pin_t*> PinList_t;
-
+typedef list<stimulus *> StimulusList_t;
+typedef list<gpsimObject *> gpsimObjectList_t;
 
 /* Support functions */
-extern void dump_node_list(void);
 extern void dump_stimulus_list(void);
 
-extern void stimuli_attach(StringList_t *);
-extern void stimuli_attach(SymbolList_t *);
-extern void stimuli_attach(Value *pNode, PinList_t *pPinList);
-
 /****************************************************************************
  *
  * Include file support stimuli. 
@@ -124,7 +84,7 @@ public:
 
   double current_time_constant; // The most recent time constant for the attached stimuli.
   double delta_voltage;     // Amplitude of initial change
-  double  minThreshold;	    // Use DC value when voltage this close
+  double  minThreshold;     // Use DC value when voltage this close
   guint64 cap_start_cycle;  // cycles when RC value last calculated
   guint64 future_cycle;     // cycles when next callback expected
 
@@ -151,8 +111,11 @@ public:
 
   void time_constant(double new_tc);
 
-  virtual void new_name(const char *);
-  virtual void new_name(string &);
+  // When a node is given a name, it is also added to the symbol
+  // table. If bClearableSymbol is true, then the symbol can be
+  // automatically removed when the symbol table is cleared.
+  virtual void new_name(const char *, bool bClearableSymbol=false);
+  virtual void new_name(string &, bool bClearableSymbol=false);
 
   // When the node is settling (due to RC charging/discharging)
   // it's voltage is periodically updated by invoking callback()
@@ -185,7 +148,7 @@ protected:
 // opamps are not modeled. In fact, even simple devices like capacitors and
 // inductors are not modeled.
 //
-class stimulus : public gpsimValue
+class stimulus : public Value
 {
 public:
 
@@ -193,13 +156,16 @@ public:
   stimulus *next;            // next stimulus that's on the snode
 
   stimulus(const char *n=0,
-	   double _Vth=5.0,
-	   double _Zth=1e3
-	   );
+           double _Vth=5.0,
+           double _Zth=1e3
+           );
   virtual ~stimulus();
 
-  virtual void new_name(const char *);
-  virtual void new_name(string &);
+  // When a stimulus is given a name, it is also added to the symbol
+  // table. If bClearableSymbol is true, then the symbol can be
+  // automatically removed when the symbol table is cleared.
+  virtual void new_name(const char *, bool bClearableSymbol=true);
+  virtual void new_name(string &, bool bClearableSymbol=true);
 
   // Functions for accessing/manipulating the thevenin voltage and impedance.
   virtual void   getThevenin(double &v, double &z, double &c);
@@ -324,14 +290,35 @@ protected:
 ///------------------------------------------------------------
 ///
 /// SignalSink - A pure virtual class that allows signals driven by external
-/// stimuli be routed to one or more objects monitoring them (e.g. one
-/// sink may be the register associated with the port while another
-/// may be a peripheral)
+/// stimuli to be routed to one or more objects monitoring them (e.g. one
+/// sink may be a bit in a port register while another may be a peripheral)
 
 class SignalSink
 {
 public:
+  virtual ~SignalSink()
+  {
+  }
+
   virtual void setSinkState(char)=0;
+  virtual void release()=0;
+};
+
+///-------------------------------------------------------------
+///
+/// AnalogSink - An analog sink is similar to a digital sink. The primary
+/// difference is that an analog sink redirects an analog signal to one
+/// or more objects. A signal sink only redirects digital signals.
+
+class AnalogSink
+{
+public:
+  virtual ~AnalogSink()
+  {
+  }
+
+  virtual void setSinkState(double)=0;
+  virtual void release()=0;
 };
 
 
@@ -349,6 +336,8 @@ public:
 
   void addSink(SignalSink *);
   void removeSink(SignalSink *);
+  void addSink(AnalogSink *);
+  void removeSink(AnalogSink *);
 
   virtual void setDrivenState(char)=0;
   virtual void setDrivingState(char)=0;
@@ -358,9 +347,12 @@ public:
   virtual void updateUI() {}  // FIXME  - make this pure virtual too.
 
 protected:
-  /// The SignalSink list is a list of all sinks that can receive data
+  /// The SignalSink list is a list of all sinks that can receive digital data
   list <SignalSink *> sinks;
 
+  /// The AnalogSink list is a list of all sinks that can receive analog data
+  list <AnalogSink *> analogSinks;
+
 };
 
 class IOPIN : public stimulus
@@ -383,11 +375,11 @@ class IOPIN : public stimulus
 
 
   IOPIN(const char *n=0,
-	double _Vth=5.0, 
-	double _Zth=1e8,
-	double _ZthWeak = 1e6,
-	double _ZthFloating = 1e7
-	);
+        double _Vth=5.0, 
+        double _Zth=1e8,
+        double _ZthWeak = 1e6,
+        double _ZthFloating = 1e7
+        );
 
   ~IOPIN();
 
@@ -421,7 +413,8 @@ class IOPIN : public stimulus
   // interface for the derived classes.
   virtual void update_direction(unsigned int x, bool refresh){ };
   virtual IOPIN_DIRECTION  get_direction(void) {return DIR_INPUT; };
-  virtual void update_pullup(char new_state, bool refresh) { }
+  virtual void update_pullup(char new_state, bool refresh) {}
+  virtual void set_is_analog(bool flag) {}
 
   virtual double get_Vth();
 
@@ -434,6 +427,7 @@ class IOPIN : public stimulus
   virtual void clr_is_newGUIname(void) { gui_name_updated = false; }
 
 protected:
+  bool   is_analog;  		  // Pin is in analog mode
   bool    gui_name_updated;       // True if object name has changed
   string  gui_name;               //
 
@@ -459,12 +453,12 @@ class IO_bi_directional : public IOPIN
 public:
 
   IO_bi_directional(const char *n=0,
-		    double _Vth=5.0, 
-		    double _Zth=150,
-		    double _ZthWeak = 1e6,
-		    double _ZthFloating = 1e7,
-		    double _VthIn = 0.3,
-		    double _ZthIn = 1e8);
+                    double _Vth=5.0, 
+                    double _Zth=150,
+                    double _ZthWeak = 1e6,
+                    double _ZthFloating = 1e7,
+                    double _VthIn = 0.3,
+                    double _ZthIn = 1e8);
   virtual double get_Zth();
   virtual double get_Vth();
   virtual char getBitChar();
@@ -492,14 +486,14 @@ class IO_bi_directional_pu : public IO_bi_directional
 {
 public:
   IO_bi_directional_pu(const char *n=0,
-		       double _Vth=5.0, 
-		       double _Zth=150,
-		       double _ZthWeak = 1e6,
-		       double _ZthFloating = 1e7,
-		       double _VthIn = 0.3,
-		       double _ZthIn = 1e8,
-		       double _Zpullup = 20e3
-		       );
+                       double _Vth=5.0, 
+                       double _Zth=150,
+                       double _ZthWeak = 1e6,
+                       double _ZthFloating = 1e7,
+                       double _VthIn = 0.3,
+                       double _ZthIn = 1e8,
+                       double _Zpullup = 20e3
+                       );
 
   ~IO_bi_directional_pu();
   virtual double get_Vth();
@@ -512,6 +506,7 @@ public:
 
   virtual char getBitChar();
   virtual void update_pullup(char new3State, bool refresh);
+  virtual void set_is_analog(bool flag);
 protected:
   bool   bPullUp;  // True when pullup is enabled
   double Zpullup;  // resistance of the pullup
