@@ -35,14 +35,14 @@ class InvalidRegister;   // Forward reference
 class _TXSTA;   // Forward references
 class _SPBRG;
 class _RCSTA;
+class _SPBRGH;
+class _BAUDCON;
 class  _14bit_processor;
 
 class RXSignalSink;
 class TXSignalSource;
 class USART_MODULE;
 
-#define TOTAL_BRGH_STATES      16
-#define TOTAL_BRGL_STATES      64
 
 
 class _TXREG : public sfr_register
@@ -50,7 +50,7 @@ class _TXREG : public sfr_register
  public:
 
 
-  _TXREG(USART_MODULE *);
+  _TXREG(Processor *pCpu, const char *pName, const char *pDesc,USART_MODULE *);
   virtual void put(unsigned int);
   virtual void put_value(unsigned int);
   virtual void assign_txsta(_TXSTA *new_txsta) { m_txsta = new_txsta; };
@@ -69,16 +69,18 @@ public:
   unsigned int bit_count;
 
   enum {
-    TX9D = 1<<0,
-    TRMT = 1<<1,
-    BRGH = 1<<2,
-    SYNC = 1<<4,
-    TXEN = 1<<5,
-    TX9  = 1<<6,
-    CSRC = 1<<7
+    TX9D  = 1<<0,
+    TRMT  = 1<<1,
+    BRGH  = 1<<2,
+    SENDB = 1<<3,
+    SYNC  = 1<<4,
+    TXEN  = 1<<5,
+    TX9   = 1<<6,
+    CSRC  = 1<<7
   };
 
-  _TXSTA(USART_MODULE *);
+  _TXSTA(Processor *pCpu, const char *pName, const char *pDesc,USART_MODULE *);
+  ~_TXSTA();
 
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
@@ -86,6 +88,7 @@ public:
   virtual void transmit_a_bit();
   virtual void start_transmitting();
   virtual void stop_transmitting();
+  virtual void transmit_break();
   virtual void callback();
   virtual void callback_print();
   virtual char getState();
@@ -94,11 +97,15 @@ public:
   virtual void putTXState(char newTXState);
 
   bool bTXEN() { return (value.get() & TXEN) != 0; }
+  
+  void set_pin_pol ( bool invert ) { bInvertPin = invert; };
+
 protected:
   USART_MODULE *mUSART;
   PinModule *m_PinModule;
   TXSignalSource *m_source;
   char m_cTxState;
+  bool bInvertPin;
 };
 
 class _RCREG : public sfr_register
@@ -111,7 +118,7 @@ class _RCREG : public sfr_register
 
   unsigned int fifo_sp;       /* fifo stack pointer */
 
-  _RCREG(USART_MODULE *);
+  _RCREG(Processor *pCpu, const char *pName, const char *pDesc,USART_MODULE *);
   virtual unsigned int get();
   virtual unsigned int get_value();
   virtual void push(unsigned int);
@@ -153,13 +160,15 @@ class _RCSTA : public sfr_register, public TriggerObject
   // produces a sample based on majority averaging. 
   // 
 
+#define TOTAL_SAMPLE_STATES    16
+
 #define BRGH_FIRST_MID_SAMPLE  4
 #define BRGH_SECOND_MID_SAMPLE 8
 #define BRGH_THIRD_MID_SAMPLE  12
 
-#define BRGL_FIRST_MID_SAMPLE  28
-#define BRGL_SECOND_MID_SAMPLE 32
-#define BRGL_THIRD_MID_SAMPLE  36
+#define BRGL_FIRST_MID_SAMPLE  7
+#define BRGL_SECOND_MID_SAMPLE 8
+#define BRGL_THIRD_MID_SAMPLE  9
 
   _RCREG  *rcreg;
   _SPBRG  *spbrg;
@@ -171,7 +180,8 @@ class _RCSTA : public sfr_register, public TriggerObject
   unsigned int sample,state, sample_state;
   guint64 future_cycle, last_cycle;
 
-  _RCSTA(USART_MODULE *);
+  _RCSTA(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *);
+  ~_RCSTA();
 
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
@@ -180,14 +190,16 @@ class _RCSTA : public sfr_register, public TriggerObject
   virtual void start_receiving();
   virtual void stop_receiving();
   virtual void overrun();
-  void set_callback_break(unsigned int spbrg_edge);
   virtual void callback();
   virtual void callback_print();
   void setState(char new_RxState);
   bool bSPEN() { return (value.get() & SPEN) != 0; }
   virtual void setIOpin(PinModule *);
+  bool rc_is_idle(void) { return ( state <= RCSTA_WAITING_FOR_START ); };
 
 protected:
+  void set_callback_break(unsigned int spbrg_edge);
+
   USART_MODULE *mUSART;
   PinModule *m_PinModule;
   RXSignalSink *m_sink;
@@ -195,18 +207,57 @@ protected:
 };
 
 
+class _BAUDCON : public sfr_register
+{
+ public:
+  enum {
+    ABDEN  = 1<<0,
+    WUE    = 1<<1,
+    BRG16  = 1<<3,
+    TXCKP  = 1<<4,
+    RXDTP  = 1<<5,
+    RCIDL  = 1<<6,
+    ABDOVF = 1<<7
+  };
+
+  _TXSTA *txsta;
+  _RCSTA *rcsta;
+
+
+  _BAUDCON(Processor *pCpu, const char *pName, const char *pDesc);
+
+  virtual void put(unsigned int);
+  virtual void put_value(unsigned int);
+  bool brg16(void) { return ( value.get() & BRG16 ) != 0; };
+
+// private:
+};
+
+class _SPBRGH : public sfr_register
+{
+ public:
+
+
+  _SPBRGH(Processor *pCpu, const char *pName, const char *pDesc);
+  virtual void assign_spbrg(_SPBRG *new_spbrg) { m_spbrg = new_spbrg; };
+ private:
+  _SPBRG   *m_spbrg;
+};
+
 class _SPBRG : public sfr_register, public TriggerObject
 {
  public:
   _TXSTA *txsta;
   _RCSTA *rcsta;
+  _SPBRGH *brgh;
+  _BAUDCON *baudcon;
 
   guint64 
     start_cycle,   // The cycle the SPBRG was started
     last_cycle,    // The cycle when the spbrg clock last changed
     future_cycle;  // The next cycle spbrg is predicted to change
 
-  _SPBRG();
+  _SPBRG(Processor *pCpu, const char *pName, const char *pDesc);
 
   virtual void callback();
   virtual void callback_print() {
@@ -216,6 +267,8 @@ class _SPBRG : public sfr_register, public TriggerObject
   virtual void get_next_cycle_break();
   virtual guint64 get_cpu_cycle(unsigned int edges_from_now);
   virtual guint64 get_last_cycle();
+// protected:
+  virtual unsigned int get_cycles_per_tick();
 };
 
 //---------------------------------------------------------------
@@ -232,7 +285,11 @@ public:
   _RCREG      *rcreg;
   PIR_SET     *pir_set;
 
-  USART_MODULE();
+  // Extra registers for when it's an EUSART
+  _SPBRGH  spbrgh;
+  _BAUDCON baudcon;
+
+  USART_MODULE(Processor *pCpu);
 
   void initialize(PIR_SET *, 
 		  PinModule *tx_pin, PinModule *rx_pin,
@@ -243,6 +300,13 @@ public:
   void full();
   void set_rcif();
   void clear_rcif();
+  bool IsEUSART ( void ) { return is_eusart; };
+  void set_eusart ( bool is_it );
+
+ private:
+  bool is_eusart;
 };
 
+
+
 #endif
